﻿/*
 * StarCoder by Tobias Wehrum <Tobias.Wehrum@dragonlab.de> is licensed under the
 * Creative Commons Attribution-ShareAlike 3.0 Unported License. To view a copy of this
 * license, visit http://creativecommons.org/licenses/by-sa/3.0/ or send a letter to
 * Creative Commons, 444 Castro Street, Suite 900, Mountain View, California, 94041, USA.
 */

using SFML.Graphics;

namespace StarCoder.Main.States
{
    // A general Game state. Examples for GameStates are the different menu screens and play modes.
    public abstract class GameState
    {
        public Game Game { get; private set; }
        public RenderWindow Window { get; private set; }

        protected GameState(Game game)
        {
            Game = game;
            Window = game.Window;
        }

        /// <summary>Called when switching to this state after the old one is teared down.</summary>
        public abstract void SetUp();

        /// <summary>Called when switching to another state before the other one is set up.</summary>
        public abstract void TearDown();

        /// <summary>Called each frame after the Game is drawn.</summary>
        public abstract void Update(float elapsedTime);

        /// <summary>Called each frame after the state is updated.</summary>
        public abstract void Draw();
    }
}
