﻿/*
 * StarCoder by Tobias Wehrum <Tobias.Wehrum@dragonlab.de> is licensed under the
 * Creative Commons Attribution-ShareAlike 3.0 Unported License. To view a copy of this
 * license, visit http://creativecommons.org/licenses/by-sa/3.0/ or send a letter to
 * Creative Commons, 444 Castro Street, Suite 900, Mountain View, California, 94041, USA.
 */

using SFML.Graphics;
using SFML.Window;

namespace StarCoder.Main.States.Play.Entities
{
    public class IngameEntity : PlayStateEntity
    {
        public uint Width { get; private set; }
        public uint Height { get; private set; }

        public Sprite sprite;

        public FloatRect Rect
        {
            get { return new FloatRect(Position.X - sprite.Origin.X, Position.Y - sprite.Origin.Y, Width, Height); }
        }

        public IngameEntity(PlayState playState, Vector2f position, string asset)
            : base(playState, position)
        {
            var texture = Game.LoadTexture(asset);

            Width = texture.Size.X;
            Height = texture.Size.Y;

            sprite = new Sprite(texture);

            Layer = PlayState.Layers.Entities;
        }

        public override void Update(float elapsedTime)
        {
        }

        public override bool Move(float dx, float dy)
        {
            var delta = playState.Level.MoveRect(Rect, new Vector2f(dx, dy));

            var result = base.Move(delta.X, delta.Y);

            WrapMovement(delta, Rect);

            return result;
        }

        public override void Draw(RenderTarget renderTarget)
        {
            sprite.Position = position;
            renderTarget.Draw(sprite);
        }
    }
}
