﻿/*
 * StarCoder by Tobias Wehrum <Tobias.Wehrum@dragonlab.de> is licensed under the
 * Creative Commons Attribution-ShareAlike 3.0 Unported License. To view a copy of this
 * license, visit http://creativecommons.org/licenses/by-sa/3.0/ or send a letter to
 * Creative Commons, 444 Castro Street, Suite 900, Mountain View, California, 94041, USA.
 */

using SFML.Graphics;
using SFML.Window;
using StarCoder.Main.Entities;

namespace StarCoder.Main.States.Play.Entities
{
    public abstract class PlayStateEntity : Entity
    {
        protected PlayState playState;

        protected PlayStateEntity(PlayState playState, Vector2f position) :
            base(playState.Game, position)
        {
            this.playState = playState;
        }

        protected void WrapMovement(Vector2f delta, FloatRect rect)
        {
            if ((delta.X < 0) && (position.X < 0))
            {
                var newRect = rect;
                newRect.Left += Constants.TARGET_WIDTH;
                var newDelta = playState.Level.MoveRect(newRect, new Vector2f(delta.X, 0));
                if (((position.X + newDelta.X) < Constants.TARGET_WIDTH) && (newDelta.X != 0))
                {
                    position.X = position.X + Constants.TARGET_WIDTH + newDelta.X;
                }
                else
                {
                    position.X = 0;
                }
            }
            else if ((delta.X > 0) && (position.X >= Constants.TARGET_WIDTH))
            {
                var newRect = rect;
                newRect.Left -= Constants.TARGET_WIDTH;
                var newDelta = playState.Level.MoveRect(newRect, new Vector2f(delta.X, 0));
                if (((position.X + newDelta.X) >= 0) && (newDelta.X != 0))
                {
                    position.X = position.X - Constants.TARGET_WIDTH + newDelta.X;
                }
                else
                {
                    position.X = Constants.TARGET_WIDTH - 1;
                }
            }

            if ((delta.Y < 0) && (position.Y < 0))
            {
                var newRect = rect;
                newRect.Top += Constants.TARGET_HEIGHT;
                var newDelta = playState.Level.MoveRect(newRect, new Vector2f(0, delta.Y));
                if (((position.Y + newDelta.Y) < Constants.TARGET_HEIGHT) && (newDelta.Y != 0))
                {
                    position.Y = position.Y + Constants.TARGET_HEIGHT + newDelta.Y;
                }
                else
                {
                    position.Y = 0;
                }
            }
            else if ((delta.Y > 0) && (position.Y >= Constants.TARGET_HEIGHT))
            {
                var newRect = rect;
                newRect.Top -= Constants.TARGET_HEIGHT;
                var newDelta = playState.Level.MoveRect(newRect, new Vector2f(0, delta.Y));
                if (((position.Y + newDelta.Y) >= 0) && (newDelta.Y != 0))
                {
                    position.Y = position.Y - Constants.TARGET_HEIGHT + newDelta.Y;
                }
                else
                {
                    position.Y = Constants.TARGET_HEIGHT - 1;
                }
            }
        }
    }
}
