﻿/*
 * StarCoder by Tobias Wehrum <Tobias.Wehrum@dragonlab.de> is licensed under the
 * Creative Commons Attribution-ShareAlike 3.0 Unported License. To view a copy of this
 * license, visit http://creativecommons.org/licenses/by-sa/3.0/ or send a letter to
 * Creative Commons, 444 Castro Street, Suite 900, Mountain View, California, 94041, USA.
 */

using System;
using System.CodeDom.Compiler;
using System.Collections.Generic;
using System.IO;
using Microsoft.CSharp;
using StarCoder.Main.Helper;

namespace StarCoder.Main.States.Play
{
    public class ProgrammablePartFactory
    {
        private CSharpCodeProvider compiler;

        public IProgrammablePart CurrentProgrammablePart { get; private set; }
        public bool HasErrors { get { return Errors.Count > 0; } }
        public List<CompilerError> Errors { get; private set; }

        public ProgrammablePartFactory()
        {
            var providerOptions = new Dictionary<string, string>();
            providerOptions.Add("CompilerVersion", "v3.5");

            compiler = new CSharpCodeProvider(providerOptions);

            CurrentProgrammablePart = new ProgrammablePart();
        }

        public bool Compile(string sourceString)
        {
            var options = new CompilerParameters();
            options.GenerateInMemory = true;
            options.ReferencedAssemblies.Add("System.dll");
            options.ReferencedAssemblies.Add("System.Core.dll");
            options.ReferencedAssemblies.Add("System.Data.dll");
            options.ReferencedAssemblies.Add("System.Xml.dll");
            options.ReferencedAssemblies.Add("mscorlib.dll");
            options.ReferencedAssemblies.Add("System.Windows.Forms.dll");
            options.ReferencedAssemblies.Add(System.Reflection.Assembly.GetExecutingAssembly().Location);

            var sfmlPath = GeneralHelper.PathCombine("libs", "SFML x86", "lib");

            options.ReferencedAssemblies.Add(Path.Combine(sfmlPath, "sfmlnet-audio-2.dll"));
            options.ReferencedAssemblies.Add(Path.Combine(sfmlPath, "sfmlnet-graphics-2.dll"));
            options.ReferencedAssemblies.Add(Path.Combine(sfmlPath, "sfmlnet-window-2.dll"));

            options.WarningLevel = 3;

            options.CompilerOptions = "/target:library /optimize";
            options.GenerateExecutable = false;
            options.GenerateInMemory = true;

            var result = compiler.CompileAssemblyFromSource(options, sourceString);

            Errors = new List<CompilerError>();
            foreach (CompilerError error in result.Errors)
            {
                Errors.Add(error);
            }

            try
            {
                CurrentProgrammablePart = (IProgrammablePart) result.CompiledAssembly.CreateInstance(typeof (ProgrammablePart).ToString());
                Program.WriteToLogFile("Compilation successful!");
                Program.WriteToLogFile();
                Program.WriteToLogFile();

                return true;
            }
            catch (Exception e)
            {
                Program.WriteToLogFile("----------------------------");
                Program.WriteToLogFile("Compilation not successful:");

                if (Constants.DEBUG_COMPILATION_OUTPUT)
                {
                    Program.WriteToLogFile("Compilation Output: ");
                    foreach (var line in result.Output)
                    {
                        Program.WriteToLogFile(line);
                    }
                    Program.WriteToLogFile();
                }
                Program.WriteToLogFile("----------------------------");
                Program.WriteToLogFile();
                Program.WriteToLogFile();

                //Console.WriteLine("While trying to set the CurrentProgrammablePart, the following exception was encountered:");
                //Console.WriteLine(e);
            }

            return false;
        }
    }
}
