﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Threading;
using System.Windows.Forms;
using StarCoder.Editor;

namespace StarCoder
{
    internal static class Program
    {
        public static StreamWriter log;

        /// <summary>
        /// Der Haupteinstiegspunkt für die Anwendung.
        /// </summary>
        [STAThread]
        private static void Main()
        {
            try
            {
                Application.EnableVisualStyles();
                Application.SetCompatibleTextRenderingDefault(false);

                StartLogFile();

                // Add the event handler for handling UI thread exceptions to the event.
                Application.ThreadException += ApplicationOnThreadException;

                // Set the unhandled exception mode to force all Windows Forms errors to go through our handler.
                Application.SetUnhandledExceptionMode(UnhandledExceptionMode.CatchException);

                // Add the event handler for handling non-UI thread exceptions to the event. 
                AppDomain.CurrentDomain.UnhandledException += CurrentDomainOnUnhandledException;

                Application.Run(new CodeEditor());
            }
            catch (Exception e)
            {
                WriteToLogFile("Exception caught in Program.Main:");
                WriteToLogFile(e.ToString());
            }
            finally
            {
                // Close the stream:
                log.Close();
                log = null;

                Application.ThreadException -= ApplicationOnThreadException;
                AppDomain.CurrentDomain.UnhandledException -= CurrentDomainOnUnhandledException;
            }
        }

        private static void StartLogFile()
        {
            if (!File.Exists("logfile.txt"))
            {
                log = new StreamWriter("logfile.txt");
            }
            else
            {
                log = File.AppendText("logfile.txt");
                log.WriteLine();
                log.WriteLine();
                log.WriteLine();
            }

            log.WriteLine("---------------------------------------------------");
            log.WriteLine("Program started at: " + DateTime.Now);
            log.WriteLine();
            log.WriteLine();
            log.WriteLine();
            log.AutoFlush = true;
            log.Flush();
        }

        private static void CurrentDomainOnUnhandledException(object sender, UnhandledExceptionEventArgs unhandledExceptionEventArgs)
        {
            WriteToLogFile("Exception caught by CurrentDomainOnUnhandledException:");
            WriteToLogFile(unhandledExceptionEventArgs.ExceptionObject.ToString());
        }

        private static void ApplicationOnThreadException(object sender, ThreadExceptionEventArgs threadExceptionEventArgs)
        {
            WriteToLogFile("Exception caught by ApplicationOnThreadException:");
            WriteToLogFile(threadExceptionEventArgs.Exception.ToString());
        }

        public static void WriteToLogFile()
        {
            WriteToLogFile("");
        }

        public static void WriteToLogFile(string str)
        {
            Console.WriteLine(str);

            if (log != null)
                log.WriteLine(str);
        }
    }
}
